Attribute VB_Name = "codeDLL"
'
'  MODULE:   ctccom32v2.bas
'
'  PURPOSE:
'    Contains declarations for Ctccom32v2 DLL.
'

' constants for binary protocol
Global Const CT_ASCII As Integer = 10
Global Const CT_REQUEST As Integer = 20
Global Const CT_REPLY As Integer = 21
Global Const CT_ACK As Integer = 100
Global Const CT_NAK As Integer = 101
Global Const CT_RANGE As Integer = 102
Global Const CT_VALUE As Integer = 104

' function return values
Global Const FAILURE As Integer = 0
Global Const SUCCESS As Integer = 1
Global Const ERROR_UNKNOWN As Integer = 255
Global Const ERROR_WIN_TIMER As Integer = 246
Global Const ERROR_NET_TIMEOUT As Integer = 236
Global Const ERROR_NET_NOTREADY As Integer = 235
Global Const ERROR_CT_RANGE As Integer = 216
Global Const ERROR_BAD_CHECKSUM As Integer = 215
Global Const ERROR_BAD_LENGTH As Integer = 214
Global Const ERROR_BAD_RESPONSE As Integer = 206
Global Const ERROR_NOT_CT_PROTOCOL As Integer = 205
Global Const ERROR_UNSUPP_REMOTE_OPER As Integer = 196
Global Const ERROR_UNKNOWN_REM_NODE As Integer = 195
Global Const ERROR_REMOTE_ACCESS As Integer = 194
Global Const ERROR_OBJECT_FILE_NOT_FOUND As Integer = 186
Global Const ERROR_FILE_WRONG_ARCH As Integer = 185
Global Const ERROR_FILE_NOT_OBJECT As Integer = 184
Global Const ERROR_BAD_OBJECT_RECORD As Integer = 183
Global Const ERROR_BAD_OBJECT_LENGTH As Integer = 182
Global Const ERROR_BAD_POSITION As Integer = 181
Global Const ERROR_EOF As Integer = 180
Global Const ERROR_BAD_PORT As Integer = 179
Global Const ERROR_NET_IN_USE As Integer = 178
Global Const ERROR_SERIAL_TIMEOUT As Integer = 170
Global Const ERROR_WARNING As Integer = 169
Global Const ERROR_ILLEGAL_BAUD_RATE As Integer = 168
Global Const ERROR_ILLEGAL_PARITY_SETTING As Integer = 167
Global Const ERROR_ILLEGAL_WORD_LENGTH As Integer = 166
Global Const ERROR_ILLEGAL_STOP_BITS As Integer = 165
Global Const ERROR_ERROR As Integer = 160
Global Const ERROR_PORT_NOT_FOUND As Integer = 159
Global Const ERROR_PORT_IN_USE As Integer = 158
    
' Constants for port types
Global Const NO_PORT As Integer = 0
Global Const SERIAL As Integer = 1
Global Const CTCNET As Integer = 2
Global Const CTCUDP As Integer = 3
Global Const CTCTCP As Integer = 4

' Constants for Parity
Global Const NO_PARITY As Byte = 0
Global Const ODD_PARITY As Byte = 1
Global Const EVEN_PARITY As Byte = 2
Global Const MARK_PARITY As Byte = 3
Global Const SPACE_PARITY As Byte = 4

' Constants for Data Bits
Global Const FOUR_DATA_BITS As Byte = 4
Global Const FIVE_DATA_BITS As Byte = 5
Global Const SIX_DATA_BITS As Byte = 6
Global Const SEVEN_DATA_BITS As Byte = 7
Global Const EIGHT_DATA_BITS As Byte = 8

' Constants for Stop Bits
Global Const ONE_STOP_BITS As Byte = 0
Global Const ONE_FIVE_STOP_BITS As Byte = 1
Global Const TWO_STOP_BITS As Byte = 2

' program file statistics
Type CT_PROG_COUNTS
    lSteps As Long
    lBytes As Long
    lErrors As Long
    lWarnings As Long
End Type

Type RANDOM_VARIANT
    register As Long
    row As Long
    col As Long
End Type


'Variant types to return and write
Global Const CT_VARIANT_INTEGER As Integer = 1
Global Const CT_VARIANT_UINTEGER As Integer = 2
Global Const CT_VARIANT_STRING As Integer = 4
Global Const CT_VARIANT_FLOAT As Integer = 8
Global Const CT_VARIANT_DOUBLE As Integer = 16

Global Const CT_MAX_STRING_SIZE As Integer = 223

'Maximum variant transfers allowed for UDP and TCP (CTNET not supported)
Global Const MAX_VARIANT_BLOCK_32BITS As Integer = 346
Global Const MAX_VARIANT_BLOCK_64BITS As Integer = 173
Global Const MAX_VARIANT_RANDOM_BLOCK As Integer = 115

'Maximum variant transfers allowed for Serial
Global Const MAX_VARIANT_BLOCK_32BITS_SERIAL As Integer = 50
Global Const MAX_VARIANT_BLOCK_64BITS_SERIAL As Integer = 25
Global Const MAX_VARIANT_RANDOM_BLOCK_SERIAL As Integer = 16

' Variant Register Structure
Type CT_VARIANT
    vRegister As Long          'Variant Register desired
    type As Long               'Format want results returned in
    precision As Long          'Precision desired for floating point conversions
    flags As Long              'Specially defined flags, 0 for normal, (indirection, etc.)
    cmd As Long                'Special commands, 0 for normal operation
    taskHandle As Long         'Alternate task handle for local task register access, 0 = default public
    slength As Long            'Length of bytes returned in stringVar, not include null
    indexCol As Long             'Column (X) selection, base 0
    indexRow As Long             'Row (X) selection base 0
    LongIntVar As Long         '32 bit signed integer storage
    FloatVar As Single         '32 bit float
    DoubleVar As Double        '64 bit double in Microsoft format
    stringVar(1 To 224) As Byte 'null terminated ASCII string of bytes
End Type

Type CT_VARIANT_BLOCK_INTEGER
    vRegister As Long          'Variant Register desired
    type As Long               'Format want results returned in
    precision As Long          'Precision desired for floating point conversions
    flags As Long              'Specially defined flags, 0 for normal, (indirection, etc.)
    cmd As Long                'Special commands, 0 for normal operation
    taskHandle As Long         'Alternate task handle for local task register access, 0 = default public
    slength As Long            'Length of bytes returned in stringVar, not include null
    indexCol As Long             'Column (X) selection, base 0
    indexRow As Long             'Row (Y) selection base 0
    numAccess As Long           'Number of items to read
    rowInc As Long              'Amount to increment row by, 0 is just read the columns
    colInc As Long              'Amount to increment column by, 0 is just read first
    arraysizeCols As Long       'Used on write operation, -1 do not expand existing columns, else columns desired.  Rows will automatically grow as needed
    ibValue(1 To 346) As Long
End Type

Type CT_VARIANT_BLOCK_DOUBLE
    vRegister As Long          'Variant Register desired
    type As Long               'Format want results returned in
    precision As Long          'Precision desired for floating point conversions
    flags As Long              'Specially defined flags, 0 for normal, (indirection, etc.)
    cmd As Long                'Special commands, 0 for normal operation
    taskHandle As Long         'Alternate task handle for local task register access, 0 = default public
    slength As Long            'Length of bytes returned in stringVar, not include null
    indexCol As Long             'Column (X) selection, base 0
    indexRow As Long             'Row (Y) selection base 0
    numAccess As Long           'Number of items to read
    rowInc As Long              'Amount to increment row by, 0 is just read the columns
    colInc As Long              'Amount to increment column by, 0 is just read first
    arraysizeCols As Long       'Used on write operation, -1 do not expand existing columns, else columns desired.  Rows will automatically grow as needed
    dbValue(1 To 173) As Double
End Type

Type CT_VARIANT_BLOCK_RANDOMDOUBLE
    vRegister As Long          'Variant Register desired
    type As Long               'Format want results returned in
    precision As Long          'Precision desired for floating point conversions
    flags As Long              'Specially defined flags, 0 for normal, (indirection, etc.)
    cmd As Long                'Special commands, 0 for normal operation
    taskHandle As Long         'Alternate task handle for local task register access, 0 = default public
    slength As Long            'Length of bytes returned in stringVar, not include null
    indexCol As Long             'Column (X) selection, base 0
    indexRow As Long             'Row (Y) selection base 0
    numAccess As Long           'Number of items to read
    rowInc As Long              'Amount to increment row by, 0 is just read the columns
    colInc As Long              'Amount to increment column by, 0 is just read first
    arraysizeCols As Long       'Used on write operation, -1 do not expand existing columns, else columns desired.  Rows will automatically grow as needed
    dbValue(1 To 115) As Double
    request(1 To 115) As RANDOM_VARIANT
End Type

' data table row structure
Global Const CT_MAX_COLS As Integer = 255
Type CT_DT_ROW
    lRow As Long
    lColumns(1 To CT_MAX_COLS) As Long
End Type

Global Const CT_MAX_RANDOM_REGS As Integer = 50
Type CT_RANDOM_REGS
    lRegList(1 To CT_MAX_RANDOM_REGS) As Long
End Type

' io count structure
Type CT_IO_COUNT
    lFlags As Long
    lDigitalInputs As Long
    lDigitalOutputs As Long
    lStepperMotors As Long
    lServoMotors As Long
    lAnalogInputs As Long
    lAnalogOutputs As Long
End Type

' misc io count structure
Type CT_MISC_IO_COUNT
    lPrototypes As Long
    lHighSpeedCounters As Long
    lThumbwheels As Long
    lDisplays As Long
End Type

' step status bank structure
Type CT_TASK_BANK
    lMode As Long
    lFaultCode As Long
    lFaultStep As Long
    lFaultData As Long
    lStep(0 To 7) As Long
    lTaskMask1(0 To 7) As Long
    lTaskMask2(0 To 7) As Long
End Type

Public Type CT_TASK_BANK2
    lMode As Long
    lFaultCode As Long
    lFaultStep As Long
    lFaultData As Long
    lTotalTasks As Long
    lStep(0 To 7) As Long
    lState(0 To 7) As Long
    lTaskMask(0 To 7, 0 To 7) As Long
End Type

' connection information structure
Type CT_CONN_INFO
    lConnType As Long
    lSrcAddr As Long
    lDestAddr As Long
    lTimeout As Long
    fConnected As Boolean
    srcEAddr(0 To 5) As Byte
    destEAddr(0 To 5) As Byte
End Type

' message information structure
Type CT_MSG_INFO
    iMsglen As Integer
    sMsgbuf As String * 256
    iRsplen As Integer
    sRspbuf As String * 256
End Type

' error information structure
Type CT_ERR_INFO
    lCode As Long
    sMessage As String * 256
End Type

' function prototypes (public)
' network access functions
Declare Function CtOpenConnection Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal CommPort As Long, ByVal Address As Long) As Long
Declare Function CtInitConnection Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal BaudRate As Long, ByVal Parity As Byte, _
    ByVal DataBits As Byte, ByVal StopBits As Byte) As Long
Declare Function CtCloseConnection Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long) As Long
Declare Function CtSetConnectionTimeout Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal MilliSeconds As Long) As Long
    
' register functions
Declare Function CtGetRegister Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal RegAddress As Long, ByRef RegValue As Long) As Long
Declare Function CtGetRegister16 Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal RegAddress As Long, ByRef Reg16Val As Long) As Long
Declare Function CtGetRegister50 Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal RegBank As Long, ByRef Reg50Val As Long) As Long
Declare Function CtGetNRegisters Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal RegCount As Long, ByRef RegList As Long, ByRef Values As Long) As Long
Declare Function CtGetAvailableAdaptersIPList Lib "Ctccom32v2.dll" _
    (ByRef IPList As Long, ByRef IPAdapters As Long, ByRef numEntries As Long) As Long
Declare Function CtPutRegister Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal RegAddress As Long, ByVal RegValue As Long) As Long
Declare Function CtGetVProperties Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal RegAddress As Long, ByRef pSizeY_Row As Long, ByRef pSizeX_Column As Long, ByRef pPrecision As Long) As Long
Declare Function CtGetVRegister Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef Storage As CT_VARIANT) As Long
Declare Function CtGetVRegisterBlockInteger Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef Storage As CT_VARIANT_BLOCK_INTEGER) As Long
Declare Function CtGetVRegisterBlockDouble Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef Storage As CT_VARIANT_BLOCK_DOUBLE) As Long
Declare Function CtGetVRegisters Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal numEntries As Long, ByRef doneEntries As Long, ByRef StorageList As CT_VARIANT) As Long
Declare Function CtSetVRegisterBlockInteger Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef Storage As CT_VARIANT_BLOCK_INTEGER) As Long
Declare Function CtSetVRegisterBlockDouble Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef Storage As CT_VARIANT_BLOCK_DOUBLE) As Long
Declare Function CtGetVRegisterRandomBlockDouble Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef Storage As CT_VARIANT_BLOCK_RANDOMDOUBLE) As Long
Declare Function CtSetVRegister Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef Storage As CT_VARIANT) As Long
Declare Function CtSetVRegisters Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal numEntries As Long, ByRef doneEntries As Long, ByRef StorageList As CT_VARIANT) As Long
Declare Function CtRunCommand Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef cmd As Byte, ByVal cmdLength As Long, ByRef result As Byte, ByRef resultLength As Long) As Long

' flag functions
Declare Function CtGetFlag Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal FlagAddress As Long, ByRef FlagValue As Long) As Long
Declare Function CtPutFlag Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal FlagAddress As Long, ByVal FlagValue As Long) As Long
    
' digital io functions
Declare Function CtGetDigitalInput8 Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Bank As Long, ByRef Value As Long) As Long
Declare Function CtGetDigitalOutput8 Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Bank As Long, ByRef Value As Long) As Long
Declare Function CtGetDigitalInput128 Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Bank As Long, ByRef Dig128Val As Long) As Long
Declare Function CtGetDigitalOutput128 Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Bank As Long, ByRef Dig128Val As Long) As Long

' analog io functions
Declare Function CtGetAnalogInput Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Index As Long, ByRef Value As Long) As Long
Declare Function CtGetAnalogOutput Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Index As Long, ByRef Value As Long) As Long
Declare Function CtPutAnalogOutput Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Index As Long, ByVal Value As Long) As Long
Declare Function CtGetAnalogInput32 Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Bank As Long, ByRef Analog32Val As Long) As Long
Declare Function CtGetAnalogOutput32 Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Bank As Long, ByRef Analog32Val As Long) As Long

' servo functions
Declare Function CtGetServoPosition Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Address As Long, ByRef Value As Long) As Long
Declare Function CtGetServoError Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Address As Long, ByRef Value As Long) As Long
Declare Function CtGetServoInputs Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Address As Long, ByRef Value As Long) As Long

' data table functions
Declare Function CtGetDataTableDim Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef NumRows As Long, ByRef numCols As Long) As Long
Declare Function CtPutDataTableDim Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal NumRows As Long, ByVal numCols As Long) As Long
Declare Function CtGetDataTableLoc Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal row As Long, ByVal col As Long, ByRef Value As Long) _
    As Long
Declare Function CtPutDataTableLoc Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal row As Long, ByVal col As Long, ByVal Value As Long) _
    As Long
Declare Function CtGetDataTableRow Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal row As Long, ByVal col As Long, ByVal numCols As Long, _
    ByRef DtRow As CT_DT_ROW) As Long
Declare Function CtPutDataTableRow Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal row As Long, ByVal col As Long, ByVal numCols As Long, _
    ByRef DtRow As CT_DT_ROW) As Long
    
' status functions
Declare Function CtGetStatus Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef CtStatus As Long) As Long
Declare Function CtPutStatus Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal CtStatus As Long) As Long
Declare Function CtGetTaskBank Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Bank As Long, ByRef TaskStatus As CT_TASK_BANK) As Long

' configuration functions
Declare Function CtGetModel Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef CtModel As Long, ByRef IsEA As Boolean) As Long
Declare Function CtGetConfig Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef CtConfig As Long) As Long
Declare Function CtPutConfig Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal CtConfig As Long) As Long
Declare Function CtGetIoCount Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef IoCounts As CT_IO_COUNT) As Long
Declare Function CtGetMiscIoCount Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef miscCounts As CT_MISC_IO_COUNT) As Long
    
' program functions
Declare Function CtDownload Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Filename As String, _
    ByRef ProgramCounts As CT_PROG_COUNTS) As Long
Declare Function CtUpload Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Filename As String, _
    ByRef ProgramCounts As CT_PROG_COUNTS) As Long

' miscellaneous functions
Declare Function CtGetConnectionInfo Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef ConnInfo As CT_CONN_INFO) As Long
Declare Function CtGetMessageInfo Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef MsgInfo As CT_MSG_INFO) As Long
Declare Function CtGetErrorInfo Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef ErrInfo As CT_ERR_INFO) As Long
Declare Function CtSendData Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal PktType As Integer, ByVal Pkt As String, _
    ByVal PktLen As Integer, ByVal ReturnPkt As String) As Long
Declare Function CtSendAscii Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByVal Pkt As String, ByVal PktLen As Integer, _
    ByVal ReturnPkt As String, ByVal termChar As Byte) As Long
Declare Function CtSetAdapter Lib "Ctccom32v2.dll" (ByVal id As Long) As Long

' devicenet / distributed module functions
Declare Function CtLoadRemoteIOConfig Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef ProgramCounts As CT_PROG_COUNTS) As Long
Declare Function CtUnloadRemoteIOConfig Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef ProgramCounts As CT_PROG_COUNTS) As Long
Declare Function CtGetRemoteIOData Lib "Ctccom32v2.dll" _
    (ByVal ConnectID As Long, ByRef ProgramCounts As CT_PROG_COUNTS) As Long

